@@ -149,13 +149,16 @@ type outputOpts struct {
 
 func main() {
 	ctx := context.Background()
-
 	opts := logger.LogOpts{
 		LoggerName:     programName,
 		FormatFunction: logFormat,
 		// No need for syslog.
 		DisableLocalLogging: true,
 	}
+	createdByBytes, err := getMetadata(ctx, "/instance/attributes/created-by")
+	if err == nil {
+		opts.MIG = string(createdByBytes)
+	}
 
 	opts.Writers = []io.Writer{os.Stderr}
 
