@@ -143,9 +143,11 @@ func main() {
 	ctx := context.Background()
 	username := os.Args[1]
 
+	createdBy, _ := client.GetKey(ctx, "/instance/attributes/created-by", nil)
 	opts := logger.LogOpts{
 		LoggerName:     programName,
 		FormatFunction: logFormat,
+		MIG:            createdBy,
 	}
 
 	if runtime.GOOS == "windows" {
@@ -154,7 +156,14 @@ func main() {
 	} else {
 		opts.Writers = []io.Writer{os.Stderr}
 	}
-	logger.Init(ctx, opts)
+
+	if err := logger.Init(ctx, opts); err != nil {
+		fmt.Printf("Error initializing logger: %+v", err)
+		os.Exit(1)
+	}
+
+	// Try flushing logs before exiting, if not flushed logs could go missing.
+	defer logger.Close()
 
 	instanceAttributes, err := getMetadataAttributes(ctx, "instance/attributes/")
 	if err != nil {
