@@ -111,7 +111,7 @@ func resetPwd(username, pwd string) error {
 	return nil
 }
 
-func addUserToGroup(ctx context.Context, username, group string) error {
+func addUserToGroup(_ context.Context, username, group string) error {
 	gPtr, err := syscall.UTF16PtrFromString(group)
 	if err != nil {
 		return fmt.Errorf("error encoding group to UTF16: %v", err)
@@ -138,7 +138,7 @@ func addUserToGroup(ctx context.Context, username, group string) error {
 	return nil
 }
 
-func createUser(ctx context.Context, username, pwd string) error {
+func createUser(_ context.Context, username, pwd, _ string) error {
 	uPtr, err := syscall.UTF16PtrFromString(username)
 	if err != nil {
 		return fmt.Errorf("error encoding username to UTF16: %v", err)
@@ -160,9 +160,14 @@ func createUser(ctx context.Context, username, pwd string) error {
 		uintptr(unsafe.Pointer(&uInfo1)),
 		uintptr(0),
 	)
-	if ret != 0 {
+
+	// If Error 2236 = The user already belongs to this group.
+	// No action is required, see:
+	// https://learn.microsoft.com/en-us/troubleshoot/windows-server/remote/terminal-server-error-messages-2200-to-2299#error-2236
+	if ret != 0 && ret != 2236 {
 		return fmt.Errorf("nonzero return code from NetUserAdd: %s", syscall.Errno(ret))
 	}
+
 	return nil
 }
 
@@ -184,6 +189,6 @@ func userExists(name string) (bool, error) {
 	return true, nil
 }
 
-func getUID(path string) string {
-	return ""
+func getUIDAndGID(_ string) (string, string) {
+	return "", ""
 }
