@@ -23,7 +23,7 @@ import (
 )
 
 // TODO: addLocalRoute and addRoute should be merged with the addition of ipForwardType to ipForwardEntry.
-func addIPForwardEntry(route ipForwardEntry) error {
+func addIPForwardEntry(ipForwardEntry) error {
 	return errors.New("addIPForwardEntry unimplemented on non Windows systems")
 }
 
@@ -32,10 +32,10 @@ func getIPForwardEntries() ([]ipForwardEntry, error) {
 	return nil, errors.New("getIPForwardEntries unimplemented on non Windows systems")
 }
 
-func addAddress(ip net.IP, mask net.IPMask, index uint32) error {
+func addAddress(net.IP, net.IPMask, uint32) error {
 	return errors.New("addAddress unimplemented on non Windows systems")
 }
 
-func removeAddress(ip net.IP, index uint32) error {
+func removeAddress(net.IP, net.IPMask, uint32) error {
 	return errors.New("removeAddress unimplemented on non Windows systems")
 }
