@@ -19,6 +19,7 @@ import (
 	"encoding/json"
 	"reflect"
 	"runtime"
+	"slices"
 	"sync/atomic"
 
 	"github.com/GoogleCloudPlatform/guest-agent/google_guest_agent/cfg"
@@ -94,7 +95,7 @@ func (d *diagnosticsMgr) Set(ctx context.Context) error {
 	}
 
 	strEntry := newMetadata.Instance.Attributes.Diagnostics
-	if utils.ContainsString(strEntry, diagnosticsEntries) {
+	if slices.Contains(diagnosticsEntries, strEntry) {
 		return nil
 	}
 	diagnosticsEntries = append(diagnosticsEntries, strEntry)
